//
// Created by philipp on 30.11.17.
//

#ifndef PROTOCOL_LEADERVEHICLE_H
#define PROTOCOL_LEADERVEHICLE_H


#include <mutex>
#include <unordered_map>
#include <bits/unordered_map.h>
#include "Vehicle.h"
#include "PlatoonService.h"
#include "Utils.h"
#include <NetworkingLib/DatagramSender.h>
#include <NetworkingLib/DatagramReceiver.h>

namespace protocol
{

class LeaderVehicle : public Vehicle
{
private:
    struct PrivateTag
    {
    };

public:
    using Ptr = std::shared_ptr<LeaderVehicle>;

    static Ptr create(networking::Networking & net, VehicleId ownVehicleId);

    // This should not be used outside.
    LeaderVehicle(PrivateTag, networking::Networking & net, VehicleId ownVehicleId);

    void stop() override;

    std::size_t getNumFollowers();

    void setPlatoonConfig(const PlatoonConfig & platoonConfig);

protected:
    void doCreatePlatoon() override;

    void doRunPlatoon() override;

    void doLeavePlatoon() override;

    void makePlatoonCreateRequest(const VehicleEndpoint & vehicleEndpoint) override;

    void makePlatoonCreateResponse(const networking::Resolver::Endpoint & clientEndpoint,
                                   const PlatoonMessage & request,
                                   PlatoonMessage & response) override;

    Ptr shared_from_this();

private:
    static constexpr networking::time::Duration HEARTBEAT_TIMEOUT = std::chrono::milliseconds(1000);
    static constexpr networking::time::Duration BROADCAST_INTERVAL = std::chrono::milliseconds(10);

    struct Follower
    {
        VehicleEndpoint endpoint;
        networking::time::Timer::Ptr heartbeatTimer;
        networking::message::DatagramSender<PlatoonMessage>::Ptr broadcastSender;
    };

    std::unordered_map<VehicleId, Follower> followers{2};
    std::mutex followersMutex;
    networking::time::Timer::Ptr broadcastTimer;
    networking::message::DatagramReceiver<PlatoonMessage>::Ptr heartbeatReceiver;

    bool findFollower(VehicleId id);

    void addFollower(const VehicleEndpoint & endpoint);

    void removeFollower(Follower & follower);

    void startHeartbeatTimer(Follower & follower);

    void stopHeartbeatTimer(Follower & follower);

    void startReceiveHeartbeats();

    void receiveHeartbeat();

    void stopReceiveHeartbeats();

    void startBroadcasting();

    void stopBroadcasting();

    void removeAllFollowers();
};

}


#endif //PROTOCOL_LEADERVEHICLE_H
