//
// Created by philipp on 02.12.17.
//

#include <iostream>
#include "ProtocolTest.h"
#include "../include/LeaderVehicle.h"
#include "../include/FollowerVehicle.h"

using namespace protocol;

void test::testBasicSetup()
{
    networking::Networking net1;
    networking::Networking net2;
    networking::Networking net3;
    using namespace protocol;

    auto leader = LeaderVehicle::create(net1, 1);
    auto follower1 = FollowerVehicle::create(net2, 2);
    auto follower2 = FollowerVehicle::create(net3, 3);

    leader->addVehicleEndpoint(2, "127.0.0.1");
    leader->addVehicleEndpoint(3, "127.0.0.1");
    follower1->addVehicleEndpoint(1, "127.0.0.1");
    follower1->addVehicleEndpoint(3, "127.0.0.1");
    follower2->addVehicleEndpoint(1, "127.0.0.1");
    follower2->addVehicleEndpoint(2, "127.0.0.1");

    leader->setPlatoonConfig(PlatoonConfig{10.0f, 20.0f});

    int configCount1 = 0;
    follower1->setOnPlatoonConfigUpdatedCallback(
        [&configCount1, follower1]
        {
            auto config = follower1->getPlatoonConfig();
            std::cout << "Follower[2] received config[" << configCount1 << "]: {ps: " << config.platoonSpeed
                      << ", ipd: " << config.innerPlatoonDistance << "}\n";
            configCount1++;
        });

    int configCount2 = 0;
    follower2->setOnPlatoonConfigUpdatedCallback(
        [&configCount2, follower2]
        {
            auto config = follower2->getPlatoonConfig();
            std::cout << "Follower[3] received config[" << configCount2 << "]: {ps: " << config.platoonSpeed
                      << ", ipd: "
                      << config.innerPlatoonDistance << "}\n";
            configCount2++;
        });

    leader->setOnRunningPlatoonCallback([]
                                        { std::cout << "Leader[1] runs the platoon\n"; });
    follower1->setOnRunningPlatoonCallback([]
                                           { std::cout << "Follower[2] runs the platoon\n"; });
    follower2->setOnRunningPlatoonCallback([]
                                           { std::cout << "Follower[3] runs the platoon\n"; });

    leader->setOnLeavingPlatoonCallback([]
                                        { std::cout << "Leader[1] leaves the platoon\n"; });
    follower1->setOnLeavingPlatoonCallback([]
                                           { std::cout << "Follower[2] leaves the platoon\n"; });
    follower2->setOnLeavingPlatoonCallback([]
                                           { std::cout << "Follower[3] leaves the platoon\n"; });

    leader->createPlatoon();
    follower1->createPlatoon();
    follower2->createPlatoon();

    sleep(1);

    leader->setPlatoonConfig(PlatoonConfig{5.0f, 10.0f});

    sleep(3);

    std::cout << "Follower[2] will now leave the platoon\n";

    follower1->leavePlatoon();

    sleep(1);

    std::cout << "Follower[3] will now leave the platoon\n";

    follower2->leavePlatoon();

    sleep(1);

    std::cout << "Leader[1] will now leave the platoon\n";

    leader->leavePlatoon();

    sleep(1);
}

void test::testStop()
{
    networking::Networking net;

    // Test if the destruction works as expected.
    {
        auto leader = LeaderVehicle::create(net, 1);
        leader->createPlatoon();
        sleep(1);
    }
    std::cout << "LEADER SUCCESS!\n";
    sleep(1);
    {
        auto follower = FollowerVehicle::create(net, 1);
        follower->createPlatoon();
        sleep(1);
    }
    std::cout << "FOLLOWER SUCCESS!\n";
}
